/**
 * k-mismatch queries
 * 
 * Aligment using rotation indexes, see README for references.
 *
 */

#ifndef _RotationQuery_H_
#define _RotationQuery_H_

#include "Pattern.h"
#include "InputReader.h"
#include "OutputWriter.h"
#include "EditDistance.h"
#include "ResultSet.h"
#include "Query.h"

#include <string>
#include <cstdlib> // exit()
#include <cmath> // pow()

class RotationQuery : public Query
{
public:
    RotationQuery(TextCollection *tc, OutputWriter &ow, bool vrb, 
                  bool incr, bool rvrs, unsigned rprt);

    void firstStep(void);
private:
    void nextStep(int pos, unsigned cnt, int k, int maxk);

    void exactMatch(int pos, unsigned cnt, int k, int maxk);

    inline void newMatch(int offset) 
    {
        ulong min = smin.top();
        ulong occs = smax.top() - min + 1;
        if (occs > report - sum)
            occs = report - sum;
        this->sum += occs;

        TextCollection::position_vector posv;
        tc->getPosition(posv, min, min + occs - 1);

        // Reconstruct text (excluding '#' marker)
        std::string text;
        text.reserve(patlen+1);
        for (int i = offset - 1; i >= 0; i--)
            text += match[i];
        for (int i = match.size() - 1; i > offset; i--) 
            text += match[i];

        for (TextCollection::position_vector::iterator pos = posv.begin(); 
             pos != posv.end(); ++pos) 
        {
            // Scale the position
            (*pos).second = (*pos).second / (2 * patlen + 1);
            outputw.reportMismatches(*p, *pos, tc->getName((*pos).first), text, 
                           smax.top() - min + 1);
        }
    }

    int psize;
    unsigned enforcedpatlen;
};

#endif // _RotationQuery_H_
